@@ -24,7 +24,6 @@ use ibc_proto::ibc::core::channel::v1::QueryConnectionChannelsRequest;
 use crate::chain::counterparty::{channel_connection_client, channel_state_on_destination};
 use crate::chain::handle::ChainHandle;
 use crate::chain::tx::TrackedMsgs;
-use crate::channel::version::ResolveContext;
 use crate::connection::Connection;
 use crate::foreign_client::{ForeignClient, HasExpiredOrFrozenError};
 use crate::object::Channel as WorkerChannelObject;
@@ -34,7 +33,7 @@ use crate::util::retry::{retry_count, RetryResult};
 use crate::util::task::Next;
 
 pub mod error;
-mod version;
+pub mod version;
 
 mod retry_strategy {
     use core::time::Duration;
@@ -398,7 +397,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
 
             from_retry_error(
                 err,
-                format!("Failed to finish channel open init for {:?}", self),
+                format!("failed to finish channel open init for {:?}", self),
             )
         })?;
 
@@ -435,7 +434,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
 
             from_retry_error(
                 err,
-                format!("Failed to finish channel open try for {:?}", self),
+                format!("failed to finish channel open try for {:?}", self),
             )
         })?;
 
@@ -609,7 +608,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
             error!("failed to open channel after {} retries", err);
             from_retry_error(
                 err,
-                format!("Failed to finish channel handshake for {:?}", self),
+                format!("failed to finish channel handshake for {:?}", self),
             )
         })?;
 
@@ -673,7 +672,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                     );
                     RetryResult::Err(index)
                 } else {
-                    error!("Failed Chan{:?} with error: {}", state, e);
+                    error!("failed Chan{:?} with error: {}", state, e);
                     RetryResult::Retry(index)
                 }
             }
@@ -717,7 +716,23 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
 
         let counterparty = Counterparty::new(self.src_port_id().clone(), None);
 
-        let version = version::resolve(ResolveContext::ChanOpenInit, self)?;
+        // If the user supplied a version, use that.
+        // Otherwise, either use the version defined for the `transfer`
+        // or an empty version if the port is non-standard.
+        let version = self
+            .dst_version()
+            .cloned()
+            .or_else(|| version::default_by_port(self.dst_port_id()))
+            .unwrap_or_else(|| {
+                warn!(
+                    chain = %self.dst_chain().id(),
+                    channel = ?self.dst_channel_id(),
+                    port = %self.dst_port_id(),
+                    "no version specified for the channel, falling back on empty version"
+                );
+
+                Version::empty()
+            });
 
         let channel = ChannelEnd::new(
             State::Init,
@@ -867,7 +882,8 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         let counterparty =
             Counterparty::new(self.src_port_id().clone(), self.src_channel_id().cloned());
 
-        let version = version::resolve(ResolveContext::ChanOpenTry, self)?;
+        // Re-use the version that was either set on ChanOpenInit or overwritten by the application.
+        let version = src_channel.version().clone();
 
         let channel = ChannelEnd::new(
             State::TryOpen,
