@@ -1,5 +1,6 @@
 #![forbid(unsafe_code)]
 #![deny(
+    warnings,
     trivial_casts,
     trivial_numeric_casts,
     unused_import_braces,
@@ -16,6 +17,8 @@
 //!
 //! [Hermes]: https://docs.rs/ibc-relayer-cli/0.2.0/
 
+extern crate alloc;
+
 pub mod chain;
 pub mod channel;
 pub mod config;
