@@ -17,7 +17,7 @@ use ibc::{
 
 use crate::{
     chain::{handle::ChainHandle, HealthCheck},
-    config::{ChainConfig, Config},
+    config::{ChainConfig, Config, SharedConfig},
     event::{
         self,
         monitor::{Error as EventError, ErrorDetail as EventErrorDetail, EventBatch},
@@ -84,7 +84,7 @@ pub struct SupervisorOptions {
    value is dropped.
 */
 pub fn spawn_supervisor(
-    config: Arc<RwLock<Config>>,
+    config: SharedConfig,
     registry: SharedRegistry<impl ChainHandle>,
     rest_rx: Option<rest::Receiver>,
     options: SupervisorOptions,
@@ -382,7 +382,7 @@ fn collect_event<F>(
     }
 }
 
-fn collect_events(
+pub fn collect_events(
     config: &Config,
     workers: &WorkerMap,
     src_chain: &impl ChainHandle,
@@ -840,7 +840,7 @@ fn update_config<Chain: ChainHandle>(
     }
 }
 
-/// Describes the result of [`collect_events`](Supervisor::collect_events).
+/// Describes the result of [`collect_events`].
 #[derive(Clone, Debug)]
 pub struct CollectedEvents {
     /// The height at which these events were emitted from the chain.
